{ nixpkgs ? import <nixpkgs> {}, compiler ? "default", doBenchmark ? false }:

let

  inherit (nixpkgs) pkgs;

  affection = with haskellPackages; callPackage(
    { mkDerivation, base, bytestring, clock, containers, glib, linear
    , monad-loops, monad-parallel, mtl, OpenGL, sdl2, stdenv, stm, text
    , uuid, vector
    }:
    mkDerivation {
      pname = "affection";
      version = "0.0.0.10";
      src = ../affection;
      isLibrary = true;
      isExecutable = true;
      libraryHaskellDepends = [
        base bytestring clock containers glib linear monad-loops
        monad-parallel mtl OpenGL sdl2 stm text uuid vector
      ];
      homepage = "https://github.com/nek0/affection#readme";
      description = "A simple Game Engine using SDL";
      license = stdenv.lib.licenses.lgpl3;
      doHaddock = false;
    }) {};

  algebraic-graphs = with haskellPackages; callPackage(
    { mkDerivation, array, base, base-compat, base-orphans, containers
    , deepseq, extra, inspection-testing, mtl, QuickCheck, stdenv
    }:
    mkDerivation {
      pname = "algebraic-graphs";
      version = "0.4";
      sha256 = "c905d32a6178a11e3c8096dbbf3bd19e570e87362c51fdc8653b43a51e46d3b7";
      libraryHaskellDepends = [
        array base base-compat containers deepseq mtl
      ];
      testHaskellDepends = [
        array base base-compat base-orphans containers extra
        inspection-testing QuickCheck
      ];
      homepage = "https://github.com/snowleopard/alga";
      description = "A library for algebraic graph construction and transformation";
      license = stdenv.lib.licenses.mit;
      doCheck = false;
    }) {};

  nanovg = with haskellPackages; callPackage(
    { mkDerivation, base, bytestring, c2hs, containers, glew
    , hspec, inline-c, libGL, libGLU, QuickCheck, stdenv, text, vector
    }:
    mkDerivation {
      pname = "nanovg";
      version = "0.6.0.0";
      sha256 = "e35a2772f2a2e71916013cab70eeb8eedd6f66b5490ddf7ce237c8c76895d496";
      revision = "1";
      editedCabalFile = "1lhfsq50r9bdby7mfscw8ih3wsz2m19lr2a2z9b6diaf5kz76vij";
      configureFlags = [ "-fgl3" ];
      isLibrary = true;
      isExecutable = true;
      libraryHaskellDepends = [ base bytestring containers text vector ];
      librarySystemDepends = [ glew libGL libGLU ];
      libraryPkgconfigDepends = [ glew ];
      libraryToolDepends = [ c2hs ];
      testHaskellDepends = [ base containers hspec inline-c QuickCheck ];
      homepage = "https://github.com/cocreature/nanovg-hs";
      description = "Haskell bindings for nanovg";
      license = stdenv.lib.licenses.isc;
      doCheck = false;
    }) {};

  f = { mkDerivation, astar, base
      , bytestring, containers, ecstasy, JuicyPixels, JuicyPixels-extra
      , linear, matrix, mtl, OpenGL, parallel, random, sdl2
      , split, stdenv, stm, text, unordered-containers, vector
      }:
      mkDerivation {
        pname = "tracer-game";
        version = "0.0.0.0";
        src = ./.;
        isLibrary = false;
        isExecutable = true;
        executableHaskellDepends = [
          affection algebraic-graphs astar base bytestring containers ecstasy
          JuicyPixels JuicyPixels-extra linear matrix mtl nanovg OpenGL
          parallel random sdl2 split stm text unordered-containers vector
        ];
        license = stdenv.lib.licenses.gpl3;
      };

  haskellPackages = if compiler == "default"
                       then pkgs.haskellPackages
                       else pkgs.haskell.packages.${compiler};

  variant = if doBenchmark then pkgs.haskell.lib.doBenchmark else pkgs.lib.id;

  drv = variant (haskellPackages.callPackage f {});

in

  if pkgs.lib.inNixShell then drv.env else drv
